package simpleex.ui;


import java.util.Iterator;

import javafx.geometry.Insets;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Region;
import javafx.scene.layout.VBox;
import simpleex.core.LatLong;
import simpleex.core.MetaData;

/**
 * The controller for the renderer in the ListView cells
 * containing the locations 
 * @author Adrian Stoica
 *
 */
public class LatLongCellController {

	/**
	 * The main container of the cell UI
	 */
	private Region root;
	
	/**
	 * the horizontal box containing the coordinates
	 * and the add/edit button 
	 */
	private HBox hBox;
	
	/**
	 * The button that will allow opening the editor 
	 * for the selected item
	 */
	private Button editMetadataButton;
	
	/**
	 * container for the additional metadata
	 */
	private VBox vBox;
	
	/**
	 * the label for the name property
	 */
	private Label nameLabel;
	
	/**
	 * the label for the coordinates
	 */
	private Label coordinatesLabel;

	/**
	 * the label for the description 
	 */
	private Label descriptionLabel;
	
	/**
	 * the current LatLong object that needs to be displayed
	 */
	private LatLong latLong;
	
	/**
	 *  create a new controller for managing  a LatLong list cell 
	 */
	public LatLongCellController() {
		super();
	}
	
	/**
	 * set the location object
	 * @param latLong the reference to the object to be displayed
	 */
	public void setLatLong(LatLong latLong) {
		this.latLong = latLong;
	}
	
	/**
	 * prepare the cell UI
	 * @param selected flag indicating that the item is selected
	 */
	protected void prepareView(boolean selected) {
		this.hBox = new HBox();
		this.coordinatesLabel = new Label(this.latLong.toString());
		coordinatesLabel.setPrefWidth(200.0);
		this.hBox.getChildren().add(coordinatesLabel);
		if(selected) {
			this.editMetadataButton = new Button("...");
			this.hBox.getChildren().add(editMetadataButton);
		} 
		
		if(this.latLong.hasMetaData()) {
			this.vBox = new VBox();
			MetaData metaData = this.latLong.getMetaData();
			if(metaData.hasProperty(MetaData.NAME_PROPERTY)) {
				nameLabel = new Label();
				nameLabel.setText(metaData.getProperty(MetaData.NAME_PROPERTY));
				vBox.getChildren().add(nameLabel);
			}
			vBox.getChildren().add(this.hBox);
			if(metaData.hasProperty(MetaData.DESCRIPTION_PROPERTY)) {
				descriptionLabel = new Label();
				descriptionLabel.setText(metaData.getProperty(MetaData.DESCRIPTION_PROPERTY));
				descriptionLabel.setWrapText(true);
				descriptionLabel.setMaxHeight(50.0);
				descriptionLabel.setManaged(true);
				descriptionLabel.setMaxWidth(200.0);
				vBox.getChildren().add(descriptionLabel);
			}
			
			final Iterator<String> tags = metaData.tags();
		      if (tags.hasNext()) {
		        HBox tagsBox = new HBox();
		        tagsBox.setSpacing(5.0);
		        while (tags.hasNext()) {
		        	Label tagLabel = new Label(tags.next());
		        	tagLabel.setStyle("-fx-background-color: #43464b; \r\n" + 
		        			"	-fx-text-fill: white;\r\n" + 
		        			"	-fx-border-radius: 3 3 3 3; \r\n" + 
		        			"    -fx-background-radius: 3 3 3 03; ");
		        	tagLabel.setPadding(new Insets(0.0,3.0,0.0,3.0));
		        	tagsBox.getChildren().add(tagLabel);
		        }
		        vBox.getChildren().add(tagsBox);
		      }
		      this.root = this.vBox;
		} else {
			this.root = this.hBox;
		}
	}
	
	/**
	 * get the UI for the cell based on selection and 
	 * the available info in the latLong object
	 * @param selected flag indicating that the item is selected
	 * @return the root container for the cell UI
	 */
	public Region getCellView(boolean selected) {
		prepareView(selected);
		return this.root;
	}
}
